@@ -9,6 +9,8 @@ import anon
 import anon.atom as anp
 import scipy.optimize
 import numpy as onp
+import matplotlib
+import matplotlib.pyplot as plt
 
 #import anabel.graphics.get_axes as get_axes
 import anon.quad
@@ -71,7 +73,11 @@ class Section:
 
 class CompositeSection(Section):
     def __init__(self, Y, DY, DZ, quad, y_shift = 0.0, mat=None):
-        nr = len(Y) # number of rectangles
+        self.Y = Y
+        self.DY = DY
+        self.DZ = DZ
+        self.quad = quad
+        self.nr = nr = len(Y) # number of rectangles
         u,du = [],[]
         for i in range(nr):
             loc, wght = quad_points(**quad[i])
@@ -80,14 +86,16 @@ class CompositeSection(Section):
 
         nIPs = [len(ui) for ui in u]
         nIP = sum(nIPs)
-
+        # Length of integration intervals in reference domain
         DU = [sum(du[i]) for i in range(nr)]
 
+        # Transform integration point locations
         yi = [float( Y[i] + DY[i]/DU[i] * u[i][j] )\
                 for i in range(nr) for j in range(nIPs[i])]
 
         dy = [float( DY[i]/DU[i] * du[i][j] )\
                 for i in range(nr) for j in range(nIPs[i])]
+
         dz = [DZ[i] for i in range(nr) for j in range(nIPs[i])]
 
         yi, dy, dz = map(list, zip( *sorted(zip(yi, dy, dz) )))
@@ -98,13 +106,62 @@ class CompositeSection(Section):
 
         yrc = sum(y*dY*dZ for y,dY,dZ in zip(Y,DY,DZ))/sum(dY*dZ for dY,dZ in zip(DY,DZ))
 
-        Izrq = sum(yi[i]**2*dA[i] for i in range(nIP)) # I   w.r.t  z @ yref using quadrature
+        Izrq = sum(yi[i]**2*dA[i] for i in range(nIP)) # I w.r.t  z @ yref using quadrature
 
         Izr =  sum(DZ[i]*DY[i]**3/12 + DZ[i]*DY[i]*(Y[i])**2 for i in range(nr))
 
         Izc =  sum(DZ[i]*DY[i]**3/12 + DZ[i]*DY[i]*(Y[i]+yrc)**2 for i in range(nr))
 
         self.SectData = {'nIP': nIP,'dA': dA,'yi': yi,'Qm':Qm, 'yrc':yrc,'Izrq':Izrq,'Izr':Izr,'Izc':Izc,'mat':mat}
+        self.__dict__.update(self.SectData)
+
+    def plot(self,
+            show_properties=True,
+            plain=False,
+            show_quad=True,
+            show_dims=True,
+            annotate=True,
+        ):
+        """Plot a composite cross section.
+
+        """
+        if plain:
+            show_properties = show_quad = show_dims = False
+        if show_properties:
+            fig = plt.figure(constrained_layout=True)
+            gs = fig.add_gridspec(1,5)
+            axp = fig.add_subplot(gs[0,3:-1])
+            label = "\n".join(["${}$:\t\t{:0.4}".format(k,v) for k,v in self.properties().items()])
+            axp.annotate(label, (0.1, 0.5), xycoords='axes fraction', va='center')
+            axp.set_autoscale_on(True)
+            axp.axis("off")
+
+            ax = fig.add_subplot(gs[0,:3])
+        else:
+            fig, ax = plt.subplots()
+
+        import matplotlib.patches as patches
+        pc = matplotlib.collections.PatchCollection(
+            [ matplotlib.patches.Rectangle((-self.DZ[i]/2.0, y-self.DY[i]/2.0), self.DZ[i], self.DY[i]) for i,y in enumerate(self.Y) ],
+            facecolor = "grey",
+            edgecolor = "grey",
+            alpha = 0.3,
+        )
+        ax.add_collection(pc)
+        if show_quad:
+            # Plot integration points
+            ax.scatter([0.0]*self.nIP,self.yi,s=[da**1.1 for da in self.dA], color="r", alpha=0.4)
+        if annotate:
+            self.annotate(ax)
+        ax.set_autoscale_on(True)
+        x_lim = 1.2*max(self.DZ)/2.0
+        ax.set_xlim(-x_lim, x_lim)
+        y_lim = 1.1*(max(self.DY)/2.0 + max(self.Y))
+        ax.set_ylim(-y_lim, y_lim)
+        ax.set_aspect(1)
+        if not show_dims:
+            ax.axis("off")
+        return fig, ax
 
 
 def Composite_Section(Y, DY, DZ, quad, y_shift = 0.0, mat=None):
@@ -154,29 +211,97 @@ class VerticalSection:
         pass
 
 class Tee(CompositeSection,VerticalSection):
-    def __init__(self, d=None, quad=None, b=None, bf=None,tf=None,tw=None,alpha=None, beta=None, yref=0.0, mat=None):
+    """
+    ![](img/sections/tee-dims.svg)
+
+    ```
+          bf
+    _____________ 
+    |           | tf
+    -------------
+        |   |
+        |   |
+        |   |
+        -----
+         tw
+    ```
+    """
+    def __init__(self, d=None, quad=None, b=None, bf=None,tf=None,tw=None,alpha=None, beta=None, yref=None, mat=None):
         """
         Parameters
-        ==========
-        tf: float
-            flange thickness
+        ----------
+        tf,tw, bf, d: float
+            shape parameters
         """
         if quad is None:
-            quad = [{"n": 8,"rule": "mid"}]*2
+            quad = [
+                {"n": 8,"rule": "gauss-legendre"},
+                {"n": 4,"rule": "gauss-legendre"}
+            ]
         if tf is None:
             tf = (1-alpha)*d 
             bf = b 
             tw = beta*b
         else:
             pass
+        self.tf = tf
+        self.bf = bf
+        self.tw = tw
+        self.area = area = bf*tf + (d - tf) *tw
+        # distance from top edge to centroid
+        self.y_centroid = y_c = 1/(2*area) * (tw*d**2 + (bf-tw)*tf**2)
+        self.y_plastic = d - area/(2*tw) if tf < area/(2.0*bf) else area/(2.0*bf)
+        # Moment of inertia
+        self.moi = bf*tf**3/12.0 + (bf*tf)*(tf*0.5*y_c)**2 + tw*(d-tf)**3/12 + tw*(d-tf)*((d+tf)*0.5-y_c)**2
 
+        if yref is None:
+            yref = (d-tf)/2 + tf - self.y_centroid
+            #yref = 0.0
+
+        #Yref = yref
         Yref = -yref
-        Y  = [ Yref,  (d-tf)/2 + tf/2+ Yref]
+        Y  = [ Yref,  (d-tf)/2 + tf/2 + Yref]
         
         DY = (d-tf, tf)
         DZ = [ tw , bf]
         
         super().__init__(Y,DY,DZ,quad,mat=mat)
+    
+    def properties(self):
+        return {
+            #"I_{xx}": None,
+            "A": self.area,
+            "I_{xx}": self.moi,
+            "y_c": self.y_centroid,
+            "y_{pna}": self.y_plastic
+        }
+    def annotate(self,ax):
+        props = dict(
+            linewidth=0.5,
+            color="k"
+        )
+        
+        tf = self.tf
+        bf = self.DZ[1]
+        bw = self.DZ[0]
+        d = sum(self.DY)
+        Y = self.Y
+        mid = sum(Y)/2
+        print(mid)
+        # bf
+        ax.plot([-bf/2, -bf/20], [(Y[1] + tf/2) * 1.2]*2, "-|", markevery=[ 0], **props)
+        ax.plot([ bf/2,  bf/20], [(Y[1] + tf/2) * 1.2]*2, "-|", markevery=[ 0], **props)
+        ax.annotate("$b_f$",[0.0, (Y[1] + tf/2) * 1.2])
+        # d
+        ax.plot([(bw+bf)/4]*2, [(Y[1] + tf/2), 0.2*(Y[1] + tf/2)], "-_", markevery=[ 0], **props)
+        ax.plot([(bw+bf)/4]*2, [0.2*(Y[0]-bw/2), Y[0]-bw/2], "-_", markevery=[-1], **props)
+        ax.annotate("$d$", [(bw+bf)/4, mid])
+        # tf
+        ax.annotate("$t_f$",[1.1*bf/2, Y[1]])
+        # bw
+        ax.plot([-bw/2, -bw/10], [(Y[0] - tf/2) * 1.2]*2, "-|", markevery=[ 0], **props)
+        ax.plot([ bw/2,  bw/10], [(Y[0] - tf/2) * 1.2]*2, "-|", markevery=[ 0], **props)
+        ax.annotate("$b_w$",[-0.05*bw, (Y[0] - tf/2) * 1.2])
 
 class Rectangle(CompositeSection,VerticalSection):
     def __init__(self, b, d, quad=None, yref=0.0,mat=None,**kwds):
@@ -204,6 +329,7 @@ class Rectangle(CompositeSection,VerticalSection):
         self.SectData = SectData
 
 
+
 def T_Sect(d, quad, b=None, bf=None,tf=None,tw=None,alpha=None, beta=None, yref=0.0, MatData=None):
     """
     ```
