@@ -1,6 +1,6 @@
 import emme as em
 
-mdl = em.Model(2,3)
+mdl = em.rModel(2,3)
 
 n1 = mdl.node('1',  0.0,  0.0)
 n2 = mdl.node('2',  0.0,  5.0)
@@ -19,12 +19,13 @@ mdl.fix(n5, ['x', 'y', 'rz'])
 n2.p['x'] = 30
 n3.p['y'] = -50
 
-mdl.numDOF()
-m, s = em.analysis.characterize(mdl)
+mdl.DOF = [[6, 7, 8],[1, 7, 2], [1, 3, 4], [1, 9, 5], [10, 9, 11]]
 
+m, s = em.analysis.characterize(mdl)
 assert m == 0
 assert s == 3
 
+
 # Define plastic capacity
 Qp_c = 150
 Qp_g = 120
@@ -43,5 +44,4 @@ c.Qp['+']['2'] = c.Qp['-']['2'] = Qp_g
 c.Qp['+']['3'] = c.Qp['-']['3'] = Qp_g
 
 lambdac, Q = em.analysis.PlasticAnalysis_wLBT(mdl)
-
 assert abs(lambdac - 2.2286) < 1e-3
\ No newline at end of file
